
# q-expansion of newform 4998.2.a.w, downloaded from the LMFDB on 06 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 4998
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-1], [1], [3], [0], [-6], [5], [-1], [-4], [0], [9], [-1], [8], [-9], [8], [3], [-12], [3], [8], [8], [12], [2], [8], [9], [6], [8], [18], [-10], [-12], [-4], [9], [-4], [18], [0], [-16], [18], [8], [5], [-16], [6], [-9], [3], [8], [-9], [26], [-21], [-7], [5], [8], [-18], [-13], [3], [-15], [26], [-9], [-30], [-24], [-15], [2], [-28], [-12], [5], [-12], [8], [6], [-4], [-9], [-34], [2], [12], [-1], [24], [3], [8], [-1], [8], [0], [-6], [32], [-30], [2], [-36], [5], [12], [11], [-19], [-15], [30], [17], [12], [-10], [-27], [-18], [23], [12], [11], [6], [24], [-18], [-4], [8], [17], [12], [12], [18], [20], [41], [12], [6], [-9], [2], [11], [-34], [-33], [20], [-34], [-30], [11], [-27], [30], [-33], [-13], [2], [-6], [24], [-1], [-24], [-46], [18], [50], [-10], [50], [24], [47], [-25], [48], [26], [6], [5], [12], [54], [-43], [-30], [-31], [-30], [-22], [-42], [44], [-51], [-4], [-12], [-22], [3], [-4], [-6], [17], [-30], [-46], [21], [-46], [18], [-18], [-24], [-28], [-12], [-48], [0], [-13], [32], [-4], [-15], [45], [50], [48], [23], [-4], [9], [20], [33], [-16], [-10], [14], [0], [-16], [36], [27], [-60], [38], [14], [-58], [30], [-4], [12], [-34], [-24], [-42], [-45], [44], [-46], [-42], [33], [-39], [-64], [-22], [-46], [57], [60], [-34], [-27], [-21], [-58], [-10], [0], [-1], [15], [-36], [-70], [14], [-18], [18], [12], [23], [29], [-30], [-25], [-51], [-1], [-39], [54], [-22], [0], [-34], [65], [50], [-30], [-4], [51], [-28], [-3], [-48], [-63], [66], [-52], [32], [-43], [30], [-6], [-43], [60], [-58], [69], [-70], [27], [3], [20], [24], [15], [14], [47], [-30], [47], [-25], [9], [26], [44], [39], [20], [0], [78], [-13], [-78], [20], [20], [11], [8], [59], [56], [-12], [-25], [29], [-51], [0], [-55], [66], [38], [-4], [12], [38], [9], [-70], [42], [3], [-36], [0], [-42], [-58], [63], [62], [42], [-54], [-10], [-31], [-6], [11], [-54], [-25], [32], [39], [50], [-48], [-31], [60], [-15], [-3], [32], [3], [-25], [24], [12], [-64], [30], [-91], [-28], [24], [-52], [-51], [35], [-73], [59], [-90], [21], [38], [45], [-40], [-60], [53], [-12], [-25], [81], [-22], [62], [-73], [54], [-69], [-22], [-54], [90], [-10], [-28], [-42], [-42], [50], [-88], [-60], [80], [-55], [-12], [24], [54], [57], [15], [8], [15], [33], [84], [-70], [-46], [21], [38], [-16], [3], [71], [24], [36], [-10], [-88], [-66], [30], [-61], [84], [-19], [30], [-42], [-97], [-30], [68], [60], [-40], [50], [41], [33], [-10], [-51], [72], [68], [24], [32], [-52], [-48], [-1], [-96], [50], [48], [-58], [21], [-60], [-61], [-1], [-36], [-28], [-54], [26], [6], [27], [-64], [-7], [-21], [-84], [-25], [-18], [-87], [78], [20], [66], [12], [74], [-84], [-72], [-42], [-70], [60]]
